<?php

App::uses('AppBehavior', 'Model/Behavior');

class InventoriesBehavior extends AppBehavior {

	/**
	 * bind the inventory model to the model
	 *
	 */
	protected function _linkModels(&$Model) {
		$Model->bindModel(
			array(
				'hasOne' => array(
					'Inventory' => array(
						'className' => 'EvInventory.Inventory',
						'foreignKey' => 'model_id',
						'conditions' => array(
							'Inventory.model' => EvClassRegistry::getNameFromModel($Model)
						)
					)
				)
			),
			false
		);
	}

	public function beforeBeforeSave(&$Model, $data, $options = array()) {
		$this->_linkModels($Model);

		$modelName = EvClassRegistry::getNameFromModel($Model);

		foreach ($data as $key => $item) {
			$data[$key]['Inventory']['model'] = $modelName;

			if (! empty($item['Inventory']['warning_action'])) {
				$data[$key]['Inventory']['warning_action'] = json_encode($item['Inventory']['warning_action']);
			}

			if (! empty($item['Inventory']['oos_action'])) {
				$data[$key]['Inventory']['oos_action'] = json_encode($item['Inventory']['oos_action']);
			}
		}

		return $data;
	}

	/**
	 * afterSave is called after a model is saved.
	 *
	 * @param Model $model Model using this behavior
	 * @param bool $created True if this save created a new record
	 * @param array $options Options passed from Model::save().
	 * @return bool
	 * @see Model::save()
	 */
	public function afterSave(Model $Model, $created, $options = array()) {
		// dispatch product saved event
		$Model->getEventManager()->dispatch(
			new CakeEvent('EvInventory.Behavior.Inventory.saved', $Model)
		);
	}

	/**
	 * beforeFind can be used to cancel find operations, or modify the query that will be executed.
	 * By returning null/false you can abort a find. By returning an array you can modify/replace the query
	 * that is going to be run.
	 *
	 * @param Model $model Model using this behavior
	 * @param array $query Data used to execute this query, i.e. conditions, order, etc.
	 * @return bool|array False or null will abort the operation. You can return an array to replace the
	 *   $query that will be eventually run.
	 */
	public function beforeFind(Model $Model, $query) {
		if (isset($query['list'])) {
			return $query;
		}

		$this->_linkModels($Model);

		$query['contain'][] = 'Inventory';

		return $query;
	}

	/**
	 * After find callback. Can be used to modify any results returned by find.
	 *
	 * @param Model $model Model using this behavior
	 * @param mixed $results The results of the find operation
	 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed An array value will replace the value of $results - any other value will be ignored.
	 */
	public function afterFind(Model $Model, $results, $primary = false) {
		foreach ($results as $key => $item) {
			if (! empty($item['Inventory']['warning_action'])) {
				$results[$key]['Inventory']['warning_action'] = json_decode($item['Inventory']['warning_action'], true);
			}

			if (! empty($item['Inventory']['oos_action'])) {
				$results[$key]['Inventory']['oos_action'] = json_decode($item['Inventory']['oos_action'], true);
			}
		}

		return $results;
	}

	/**
	 * method called by the reduceStock listener
	 *
	 * @param 	object 	$Model 		The model calling this behavior
	 * @param 	int 	$itemId 	The model ID to look for an update
	 * @param 	int 	$quantity 	The quantity to remove
	 * @return 	bool
	 */
	public function reduceStock($Model, $modelId, $quantity) {
		$modelName = EvClassRegistry::getNameFromModel($Model);

		$Inventory = EvClassRegistry::init('EvInventory.Inventory');

		return $Inventory->updateAll(
			array(
				'Inventory.stock' => 'Inventory.stock - ' . $quantity
			),
			array(
				'Inventory.model' => $modelName,
				'Inventory.model_id' => $modelId
			)
		);
	}
}
